use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize)]
pub struct Entry {
    pub name: String,
    pub address: Vec<String>,
    pub players_online: usize,
    pub last_game: i64,
    pub version: (u32, u32),
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct Submission {
    pub secret: u128,
    pub name: String,
    pub players: usize,
    pub last_game: i64,
    pub version: (u32, u32),
    pub uri: String,
}
